@@ -15,10 +15,11 @@
 #ifndef SOURCE_FUZZ_DATA_DESCRIPTOR_H_
 #define SOURCE_FUZZ_DATA_DESCRIPTOR_H_
 
-#include "source/fuzz/protobufs/spirvfuzz_protobufs.h"
-
+#include <ostream>
 #include <vector>
 
+#include "source/fuzz/protobufs/spirvfuzz_protobufs.h"
+
 namespace spvtools {
 namespace fuzz {
 
@@ -27,13 +28,21 @@ namespace fuzz {
 protobufs::DataDescriptor MakeDataDescriptor(uint32_t object,
                                              std::vector<uint32_t>&& indices);
 
+// Hash function for data descriptors.
+struct DataDescriptorHash {
+  size_t operator()(const protobufs::DataDescriptor* data_descriptor) const;
+};
+
 // Equality function for data descriptors.
 struct DataDescriptorEquals {
   bool operator()(const protobufs::DataDescriptor* first,
                   const protobufs::DataDescriptor* second) const;
 };
 
+std::ostream& operator<<(std::ostream& out,
+                         const protobufs::DataDescriptor& data_descriptor);
+
 }  // namespace fuzz
 }  // namespace spvtools
 
-#endif  // #define SOURCE_FUZZ_DATA_DESCRIPTOR_H_
+#endif  // SOURCE_FUZZ_DATA_DESCRIPTOR_H_
