@@ -13,6 +13,7 @@
 // limitations under the License.
 
 #include "source/fuzz/transformation_add_constant_composite.h"
+
 #include "test/fuzz/fuzz_test_util.h"
 
 namespace spvtools {
@@ -63,45 +64,68 @@ TEST(TransformationAddConstantCompositeTest, BasicTest) {
   const auto context = BuildModule(env, consumer, shader, kFuzzAssembleOption);
   ASSERT_TRUE(IsValid(env, context.get()));
 
-  FactManager fact_manager;
+  FactManager fact_manager(context.get());
   spvtools::ValidatorOptions validator_options;
   TransformationContext transformation_context(&fact_manager,
                                                validator_options);
 
   // Too few ids
-  ASSERT_FALSE(TransformationAddConstantComposite(103, 8, {100, 101})
+  ASSERT_FALSE(TransformationAddConstantComposite(103, 8, {100, 101}, false)
                    .IsApplicable(context.get(), transformation_context));
   // Too many ids
-  ASSERT_FALSE(TransformationAddConstantComposite(101, 7, {14, 15, 14})
+  ASSERT_FALSE(TransformationAddConstantComposite(101, 7, {14, 15, 14}, false)
                    .IsApplicable(context.get(), transformation_context));
   // Id already in use
-  ASSERT_FALSE(TransformationAddConstantComposite(40, 7, {11, 12})
+  ASSERT_FALSE(TransformationAddConstantComposite(40, 7, {11, 12}, false)
                    .IsApplicable(context.get(), transformation_context));
   // %39 is not a type
-  ASSERT_FALSE(TransformationAddConstantComposite(100, 39, {11, 12})
+  ASSERT_FALSE(TransformationAddConstantComposite(100, 39, {11, 12}, false)
                    .IsApplicable(context.get(), transformation_context));
 
   TransformationAddConstantComposite transformations[] = {
       // %100 = OpConstantComposite %7 %11 %12
-      TransformationAddConstantComposite(100, 7, {11, 12}),
+      TransformationAddConstantComposite(100, 7, {11, 12}, false),
 
       // %101 = OpConstantComposite %7 %14 %15
-      TransformationAddConstantComposite(101, 7, {14, 15}),
+      TransformationAddConstantComposite(101, 7, {14, 15}, false),
 
       // %102 = OpConstantComposite %7 %17 %18
-      TransformationAddConstantComposite(102, 7, {17, 18}),
+      TransformationAddConstantComposite(102, 7, {17, 18}, false),
 
       // %103 = OpConstantComposite %8 %100 %101 %102
-      TransformationAddConstantComposite(103, 8, {100, 101, 102}),
+      TransformationAddConstantComposite(103, 8, {100, 101, 102}, false),
 
       // %104 = OpConstantComposite %24 %29 %30 %31
-      TransformationAddConstantComposite(104, 24, {29, 30, 31}),
+      TransformationAddConstantComposite(104, 24, {29, 30, 31}, false),
 
       // %105 = OpConstantComposite %26 %104 %33
-      TransformationAddConstantComposite(105, 26, {104, 33}),
+      TransformationAddConstantComposite(105, 26, {104, 33}, false),
 
       // %106 = OpConstantComposite %35 %38 %39 %40
-      TransformationAddConstantComposite(106, 35, {38, 39, 40})};
+      TransformationAddConstantComposite(106, 35, {38, 39, 40}, false),
+
+      // Same constants but with an irrelevant fact applied.
+
+      // %107 = OpConstantComposite %7 %11 %12
+      TransformationAddConstantComposite(107, 7, {11, 12}, true),
+
+      // %108 = OpConstantComposite %7 %14 %15
+      TransformationAddConstantComposite(108, 7, {14, 15}, true),
+
+      // %109 = OpConstantComposite %7 %17 %18
+      TransformationAddConstantComposite(109, 7, {17, 18}, true),
+
+      // %110 = OpConstantComposite %8 %100 %101 %102
+      TransformationAddConstantComposite(110, 8, {100, 101, 102}, true),
+
+      // %111 = OpConstantComposite %24 %29 %30 %31
+      TransformationAddConstantComposite(111, 24, {29, 30, 31}, true),
+
+      // %112 = OpConstantComposite %26 %104 %33
+      TransformationAddConstantComposite(112, 26, {104, 33}, true),
+
+      // %113 = OpConstantComposite %35 %38 %39 %40
+      TransformationAddConstantComposite(113, 35, {38, 39, 40}, true)};
 
   for (auto& transformation : transformations) {
     ASSERT_TRUE(
@@ -110,6 +134,14 @@ TEST(TransformationAddConstantCompositeTest, BasicTest) {
   }
   ASSERT_TRUE(IsValid(env, context.get()));
 
+  for (uint32_t id = 100; id <= 106; ++id) {
+    ASSERT_FALSE(fact_manager.IdIsIrrelevant(id));
+  }
+
+  for (uint32_t id = 107; id <= 113; ++id) {
+    ASSERT_TRUE(fact_manager.IdIsIrrelevant(id));
+  }
+
   std::string after_transformation = R"(
                OpCapability Shader
           %1 = OpExtInstImport "GLSL.std.450"
@@ -149,6 +181,13 @@ TEST(TransformationAddConstantCompositeTest, BasicTest) {
         %104 = OpConstantComposite %24 %29 %30 %31
         %105 = OpConstantComposite %26 %104 %33
         %106 = OpConstantComposite %35 %38 %39 %40
+        %107 = OpConstantComposite %7 %11 %12
+        %108 = OpConstantComposite %7 %14 %15
+        %109 = OpConstantComposite %7 %17 %18
+        %110 = OpConstantComposite %8 %100 %101 %102
+        %111 = OpConstantComposite %24 %29 %30 %31
+        %112 = OpConstantComposite %26 %104 %33
+        %113 = OpConstantComposite %35 %38 %39 %40
           %4 = OpFunction %2 None %3
           %5 = OpLabel
                OpReturn
